import time

class LcdApi:

    LCD_CLR = 0x01              # DB0: clear display
    LCD_HOME = 0x02             # DB1: return to home position

    LCD_ENTRY_MODE = 0x04       # DB2: set entry mode
    LCD_ENTRY_INC = 0x02        # --DB1: increment
    LCD_ENTRY_SHIFT = 0x01      # --DB0: shift

    LCD_ON_CTRL = 0x08          # DB3: turn lcd/cursor on
    LCD_ON_DISPLAY = 0x04       # --DB2: turn display on
    LCD_ON_CURSOR = 0x02        # --DB1: turn cursor on
    LCD_ON_BLINK = 0x01         # --DB0: blinking cursor

    LCD_MOVE = 0x10             # DB4: move cursor/display
    LCD_MOVE_DISP = 0x08        # --DB3: move display (0-> move cursor)
    LCD_MOVE_RIGHT = 0x04       # --DB2: move right (0-> left)

    LCD_FUNCTION = 0x20         # DB5: function set
    LCD_FUNCTION_8BIT = 0x10    # --DB4: set 8BIT mode (0->4BIT mode)
    LCD_FUNCTION_2LINES = 0x08  # --DB3: two lines (0->one line)
    LCD_FUNCTION_10DOTS = 0x04  # --DB2: 5x10 font (0->5x7 font)
    LCD_FUNCTION_RESET = 0x30   # See Initializing by Instruction section

    LCD_CGRAM = 0x40            # DB6: set CG RAM address
    LCD_DDRAM = 0x80            # DB7: set DD RAM address

    LCD_RS_CMD = 0
    LCD_RS_DATA = 1

    LCD_RW_WRITE = 0
    LCD_RW_READ = 1

    def __init__(self, num_lines, num_columns):
        self.num_lines = num_lines
        if self.num_lines > 4:
            self.num_lines = 4
        self.num_columns = num_columns
        if self.num_columns > 40:
            self.num_columns = 40
        self.cursor_x = 0
        self.cursor_y = 0
        self.implied_newline = False
        self.backlight = True
        self.display_off()
        self.backlight_on()
        self.clear()
        self.hal_write_command(self.LCD_ENTRY_MODE | self.LCD_ENTRY_INC)
        self.hide_cursor()
        self.display_on()

    def clear(self):
        self.hal_write_command(self.LCD_CLR)
        self.hal_write_command(self.LCD_HOME)
        self.cursor_x = 0
        self.cursor_y = 0

    def show_cursor(self):
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def hide_cursor(self):
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def blink_cursor_on(self):
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR | self.LCD_ON_BLINK)

    def blink_cursor_off(self):
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY |
                               self.LCD_ON_CURSOR)

    def display_on(self):
        self.hal_write_command(self.LCD_ON_CTRL | self.LCD_ON_DISPLAY)

    def display_off(self):
        self.hal_write_command(self.LCD_ON_CTRL)

    def backlight_on(self):
        self.backlight = True
        self.hal_backlight_on()

    def backlight_off(self):
        self.backlight = False
        self.hal_backlight_off()

    def move_to(self, cursor_x, cursor_y):
        self.cursor_x = cursor_x
        self.cursor_y = cursor_y
        addr = cursor_x & 0x3f
        if cursor_y & 1:
            addr += 0x40    # Lines 1 & 3 add 0x40
        if cursor_y & 2:    # Lines 2 & 3 add number of columns
            addr += self.num_columns
        self.hal_write_command(self.LCD_DDRAM | addr)

    def putchar(self, char):
        self.hal_write_data(ord(char))
        self.cursor_x += 1
        if self.cursor_x >= self.num_columns:
            self.cursor_x = 0
            self.cursor_y += 1
        if self.cursor_y >= self.num_lines:
            self.cursor_y = 0
        self.move_to(self.cursor_x, self.cursor_y)

    def putstr(self, string):
        for char in string:
            self.putchar(char)

    def custom_char(self, location, charmap):
        location &= 0x7
        self.hal_write_command(self.LCD_CGRAM | (location << 3))
        self.hal_sleep_us(40)
        for i in range(8):
            self.hal_write_data(charmap[i])
            self.hal_sleep_us(40)
        self.move_to(self.cursor_x, self.cursor_y)

    def hal_backlight_on(self):
        pass

    def hal_backlight_off(self):
        pass

    def hal_write_command(self, cmd):
        raise NotImplementedError

    def hal_write_data(self, data):
        raise NotImplementedError

    def hal_sleep_us(self, usecs):
        time.sleep_us(usecs)





import urequests
import ujson
import network
import time
import machine
class eduCOBOT_IOT:
    def __init__(self,secretKey,APIKey,wifiName='eduCOBOT',wifiPassword='1234554322'):
        self.IndicatorPin = machine.Pin(2, machine.Pin.OUT)
        self.secretKey = secretKey
        self.APIKey = APIKey
        self.wifiName = wifiName
        self.wifiPassword = wifiPassword
        self.isVerified = False
        self.wlan = network.WLAN(network.STA_IF)
        self.machineURL = 'http://machine.server.iot.educobot.com'
        self.connectToWiFi()
        self.validateProject()
        self.IPAddress='0.0.0.0'
    def connectToWiFi(self):
        self.wlan.active(True)
        print('Attempting to connect to the network...')
        self.wlan.connect(self.wifiName, self.wifiPassword)        
        max_wait = 10
        while max_wait > 0 and not self.wlan.isconnected():
            max_wait -= 1
            print('waiting for connection...')
            time.sleep(1)
        if not self.wlan.isconnected():
            print('Network Connection has failed')
        else:
            print('Connected to the network successfully.')
            self.IndicatorPin.value(1)
            time.sleep(0.2)
            self.IndicatorPin.value(0)
            time.sleep(0.2)
            self.IndicatorPin.value(1)
            time.sleep(0.2)
            self.IndicatorPin.value(0)
            status = self.wlan.ifconfig()
            print( 'Assigned Local IP = ' + status[0] )
            self.IPAddress=status[0]
    def validateProject(self):
        print('verifying project....')
        try:
            response = urequests.post(
                url=self.machineURL+'/VerifyHardwareProjectAPI',
                headers = {'content-type': 'application/json'},
                data=ujson.dumps({'secretKey':self.secretKey,'APIKey':self.APIKey})
                )
            json=response.json()
            if(json['success']==True):
                print(json['message'])
                self.isVerified = True
                self.IndicatorPin.value(1)
            else:
                print(json['message'])
        except Exception as e:
            print(e)
            print('Project verification failed')
    def sendData(self,variableName,variableValue):
        if(self.isVerified):
            try:
                response = urequests.post(
                    url=self.machineURL+'/ChangeVariableValueAPI',
                    headers = {'content-type': 'application/json'},
                    data=ujson.dumps({'secretKey':self.secretKey,'APIKey':self.APIKey,'variableName':variableName,'variableValue':variableValue})
                    )
                json=response.json()
                if(json['success']==True):
                    print('Data sent successfully')
                else:print(json['message'])
            except Exception as e:
                print(e)
                pass
        else:
            print('Project not verified')
    def readData(self,variableName):
        if(self.isVerified):
            try:
                response = urequests.post(
                    url=self.machineURL+'/GetVariableValueAPI',
                    headers = {'content-type': 'application/json'},
                    data=ujson.dumps({'secretKey':self.secretKey,'APIKey':self.APIKey,'variableName':variableName})
                    )
                json=response.json()
                if(json['success']==True):
                    return json['data']['value']
                else:
                    return None
            except Exception as e:
                print(e)
                return None
        else:
            print('Project not verified')
            return None




import tkinter as tk
from tkinter import messagebox
import requests
import json
from PIL import Image, ImageTk
import os
class LoginApp:
    def __init__(self, root=None):
        if root is None:
            return
        self.root = root
        self.root.title("Innov8r Login")
        self.root.geometry("600x500")  # Updated screen size
        self.root.configure(bg="#e6f7ff")  # Softer blue background
        
        # Load and add the image at the top
        self.image = Image.open(os.path.join(os.path.dirname(__file__), "res", "innovator.png"))  # Replace with your image path
        self.image = self.image.resize((120, 120), Image.Resampling.LANCZOS)  # Updated for compatibility with new Pillow version
        self.photo = ImageTk.PhotoImage(self.image)
        self.image_label = tk.Label(self.root, image=self.photo, bg="#e6f7ff")
        self.image_label.pack(pady=(20, 10))  # Padding above and below the image

        # Main Frame
        self.main_frame = tk.Frame(self.root, bg="#e6f7ff", padx=40, pady=40)
        self.main_frame.pack(expand=True)

        # Email Label and Entry with horizontal and vertical padding
        self.email_label = tk.Label(self.main_frame, text="Email:", font=("Verdana", 14, "bold"), bg="#e6f7ff", fg="#333", anchor="w", justify="left")
        self.email_label.pack(fill="x", pady=(0, 5))
        self.email_entry = tk.Entry(self.main_frame, font=("Verdana", 12), width=30, highlightbackground="#4CAF50", highlightthickness=2, relief="groove", bd=0)
        self.email_entry.pack(ipadx=10, ipady=5, pady=(0, 15))  # Added ipadx for horizontal padding

        # Password Label and Entry with horizontal and vertical padding
        self.password_label = tk.Label(self.main_frame, text="Password:", font=("Verdana", 14, "bold"), bg="#e6f7ff", fg="#333", anchor="w", justify="left")
        self.password_label.pack(fill="x", pady=(10, 5))
        self.password_entry = tk.Entry(self.main_frame, font=("Verdana", 12), show="*", width=30, highlightbackground="#4CAF50", highlightthickness=2, relief="groove", bd=0)
        self.password_entry.pack(ipadx=10, ipady=5, pady=(0, 20))  # Added ipadx for horizontal padding

        # Login Button
        self.login_button = tk.Button(self.main_frame, 
                                       text="Login", 
                                       font=("Verdana", 12, "bold"), 
                                       bg="#0288d1",  # Stylish blue button
                                       fg="white", 
                                       activebackground="#0277bd", 
                                       activeforeground="white", 
                                       width=20, 
                                       relief="flat")  # Flat style for a modern look
        self.login_button.pack()
        self.login_button.bind("<Button-1>", self.verify_login)
    def clear_placeholder(self, event):
        widget = event.widget
        if widget.get() == "Enter your email" or widget.get() == "Enter your password":
            widget.delete(0, "end")
    def verify_login(self,event):
        email = self.email_entry.get()
        password = self.password_entry.get()
        if(email == "" or password == ""):
            messagebox.showerror("Error", "Please enter email and password.")
            return
        try:
            response = requests.post("https://backend.educobot.com/users/getUser", json={"email": email, "password": password})
            response = response.json()
            print(response)

            if response["userID"]:
                self.save_credentials(response["userID"])
                messagebox.showinfo("Success", "Login successful!", icon='info')
                self.root.destroy()
            elif response["user"]:
                self.save_credentials(response["user"])
                messagebox.showinfo("Success", "Login successful!", icon='info')
                self.root.destroy()
            else:
                messagebox.showerror("Error", "Invalid email or password")
                self.password_entry.delete(0, tk.END)

        except Exception as exception:
            print(exception)
            messagebox.showerror("Error", "Invalid email or password.")
    def save_credentials(self, userId):
        try:
            response = requests.post("https://backend.educobot.com/blockly/get-user-details", json={"userId": userId})
            response = response.json()
            print(response)
            if response["user"]:
                with open("credential.json", "w") as f:
                    json.dump(response, f)

        except Exception as exception:
            print(exception)
            messagebox.showerror("Error", "Invalid email or password.")
    def isLoggedIn(self):
        try:
            with open("credential.json", "r") as f:
                data = json.load(f)
            return True
        except:
            return False

def openLoginScreen(successFunction):
    root = tk.Tk()
    app = LoginApp(root)
    root.mainloop()
    if(app.isLoggedIn()):
        successFunction()

if __name__ == "__main__":
    root = tk.Tk()
    app = LoginApp(root)
    root.mainloop()



from flask import Flask, request, jsonify
from flask_cors import CORS
from threading import Thread
from innov8r import get_runner, get_workbench, get_shell


FlaskAPP =Flask(__name__)

CORS(FlaskAPP, resources={r"/*": {"origins": "*"}})

@FlaskAPP.route('/', methods=['GET'])
def home():
    return "it works"

@FlaskAPP.route('/api/v1/esp32/deploy', methods=['POST'])
def deploy():
    data = request.get_json()
    code = data['data']
    get_workbench().get_editor_notebook().get_current_editor().get_code_view().set_content(code)
    get_runner().cmd_run_current_script()
    return jsonify({
        'status': 'success',
        'message': 'Reached to IDE successfully'
    })

@FlaskAPP.route('/ide/shell/get/messages', methods=['GET'])
def get_shell_messages():
    messages=get_shell().text.get("1.0", "end-1c")
    # print(messages)
    return jsonify({
        'status': 'success',
        'messages': messages
    })

@FlaskAPP.route('/ide/shell/restart-backend', methods=['GET'])
def restart_shell():
    get_runner().restart_backend(clean=True)
    return jsonify({
        'status': 'success',
        'message': 'Shell restarted successfully'
    })

@FlaskAPP.route('/ide/shell/save-in-persistent-storage', methods=['POST'])
def save_in_persistent_storage():
    data = request.get_json()
    code = data['data']
    code = code.replace("\"", "\\\"")
    new_code = f"""bootPyContent = \"\"\"# This file is executed on every boot (including wake-boot from deepsleep)
#import esp
#esp.osdebug(None)
#import webrepl
#webrepl.start()
import program
\"\"\"

code = \"\"\"{code}\"\"\"

with open("boot.py", "w") as f:
    f.write(bootPyContent)
with open("program.py", "w") as f:
    f.write(code)"""
    get_workbench().get_editor_notebook().get_current_editor().get_code_view().set_content(new_code)
    get_runner().cmd_run_current_script()
    print(new_code)
    return jsonify({
        'status': 'success',
        'message': 'Code saved in persistent storage successfully'
    })
def run_app():
    FlaskAPP.run(host='0.0.0.0', port=8080, debug=False, use_reloader=False)



def startFlaskApp():
    flask_thread = Thread(target=run_app)
    flask_thread.daemon = True
    flask_thread.start()

if __name__ == '__main__':
    FlaskAPP.run(debug=True)






# Rui Santos & Sara Santos - Random Nerd Tutorials
# Complete project details at https://RandomNerdTutorials.com/raspberry-pi-pico-servo-motor-micropython/

from machine import Pin, PWM

class Servo:
    __servo_pwm_freq = 50
    __min_u16_duty = 1802
    __max_u16_duty = 7864
    min_angle = 0
    max_angle = 180
    current_angle = 0.001


    def __init__(self, pin):
        self.__initialise(pin)


    def update_settings(self, servo_pwm_freq, min_u16_duty, max_u16_duty, min_angle, max_angle, pin):
        self.__servo_pwm_freq = servo_pwm_freq
        self.__min_u16_duty = min_u16_duty
        self.__max_u16_duty = max_u16_duty
        self.min_angle = min_angle
        self.max_angle = max_angle
        self.__initialise(pin)


    def move(self, angle):
        # round to 2 decimal places, so we have a chance of reducing unwanted servo adjustments
        angle = round(angle, 2)
        # do we need to move?
        if angle == self.current_angle:
            return
        self.current_angle = angle
        # calculate the new duty cycle and move the motor
        duty_u16 = self.__angle_to_u16_duty(angle)
        self.__motor.duty_u16(duty_u16)
    
    def stop(self):
        self.__motor.deinit()
    
    def get_current_angle(self):
        return self.current_angle

    def __angle_to_u16_duty(self, angle):
        return int((angle - self.min_angle) * self.__angle_conversion_factor) + self.__min_u16_duty


    def __initialise(self, pin):
        self.current_angle = -0.001
        self.__angle_conversion_factor = (self.__max_u16_duty - self.__min_u16_duty) / (self.max_angle - self.min_angle)
        self.__motor = PWM(Pin(pin))
        self.__motor.freq(self.__servo_pwm_freq)


from innov8r import launch, report_time
from innov8r.flask_app import startFlaskApp
from innov8r.login_validator import openLoginScreen, LoginApp


startFlaskApp()
report_time("Before launch")
launch()
